local _G = _G
local Cartographer3 = _G.Cartographer3
local Cartographer3_Data = Cartographer3.Data

local battlegroundHalfways = {
	{ -- Horde
		[11] = 242.5, -- tower
		[19] = 62.5, -- mine
		[24] = 62.5, -- lumber mill
		[29] = 62.5, -- blacksmith
		[34] = 62.5, -- farm
		[39] = 62.5, -- stables
	},
	{ -- Alliance
		[8] = 242.5, -- tower
		[17] = 62.5, -- mine
		[22] = 62.5, -- lumber mill
		[27] = 62.5, -- blacksmith
		[32] = 62.5, -- farm
		[37] = 62.5, -- stables
	}
}
local battlegroundFulls = {
	{ -- Horde
		[9] = true, -- tower
		[20] = true, -- mine
		[25] = true, -- limber mill
		[30] = true, -- blacksmith
		[35] = true, -- farm
		[40] = true, -- stables
	},
	{ -- Alliance
		[10] = true, -- tower
		[18] = true, -- mine
		[23] = true, -- lumber mill
		[28] = true, -- blacksmith
		[33] = true, -- farm
		[38] = true, -- stables
	}
}

local pois = {}

local proportion = 0.75

local function makePoi()
	local poi = CreateFrame("Frame", Cartographer3_Data.mapView:GetName() .. "_Poi" .. (#pois + 1), Cartographer3_Data.mapView)
	pois[#pois+1] = poi
	poi:SetWidth(1)
	poi:SetHeight(1)
	Cartographer3.AddPOI(poi)
	function poi:AddDataToFullTooltip()
		WorldMapTooltip:SetText(self.name)
		WorldMapTooltip:AddLine(self.description, nil, nil, nil, true)
	end
	function poi:AddDataToTooltipLine()
		WorldMapTooltip:SetText(self.name)
	end

	local texture = poi:CreateTexture(nil--[[poi:GetName() .. "_Texture"]], "BORDER")
	poi.texture = texture
	texture:SetAllPoints(poi)
	texture:SetTexture([=[Interface\Minimap\POIIcons]=])
	
	local timer = poi:CreateTexture(nil--[[poi:GetName() .. "_Timer"]], "BACKGROUND")
	poi.timer = timer
	timer:SetTexture(1, 1, 1, 0.5)
	timer:SetWidth(0)
	timer.percent = 0
	function timer:Update()
		local size = poi:GetHeight()
		self:SetPoint("TOPLEFT", poi, "CENTER", -size * proportion, size * proportion)
		self:SetPoint("BOTTOMRIGHT", poi, "CENTER", (-size + size*2*self.percent) * proportion, -size * proportion)
	end
	
	poi:SetScript("OnSizeChanged", function(self)
		self.timer:Update()
	end)
	
	return poi
end

local runningTimers = {}
local runningTimersMax = {}

local function f()
	Cartographer3.Utils.RemoveTimer(f)
	
	local numPois = GetNumMapLandmarks()
	
	for i = 1, numPois do
		local poi = pois[i] or makePoi()
		
		local name, description, textureIndex, x, y = GetMapLandmarkInfo(i)
		if textureIndex ~= 15 then
			local x1, x2, y1, y2 = WorldMap_GetPOITextureCoords(textureIndex)
			poi.texture:SetTexCoord(
				(textureIndex % 8) / 8,
				((textureIndex % 8) + 1) / 8,
				math.floor(textureIndex / 8) / 8,
				(math.floor(textureIndex / 8) + 1) / 8)
		
			local poiX, poiY = Cartographer3.Utils.ConvertZoneCoordinateToUniverseCoordinate(Cartographer3_Data.currentMapTexture, x, y)
			poi:SetPoint("CENTER", Cartographer3_Data.mapView, "CENTER", poiX, poiY)
			poi.name = name
			poi.description = description
			poi:Show()
			local inInstance, kind = IsInInstance()
			if inInstance and kind == "pvp" then
				if battlegroundHalfways[1][textureIndex] then
					if not runningTimers[poi] then
						runningTimers[poi] = 0
						runningTimersMax[poi] = battlegroundHalfways[1][textureIndex]
					end
					poi.timer:SetTexture(1, 0, 0, 0.5)
				elseif battlegroundHalfways[2][textureIndex] then
					if not runningTimers[poi] then
						runningTimers[poi] = 0
						runningTimersMax[poi] = battlegroundHalfways[2][textureIndex]
					end
					poi.timer:SetTexture(0, 0, 1, 0.5)
				elseif battlegroundFulls[1][textureIndex] then
					runningTimers[poi] = nil
					runningTimersMax[poi] = nil
					poi.timer:SetTexture(1, 0, 0, 0.5)
					poi.timer.percent = 1
					poi.timer:Update()
				elseif battlegroundFulls[2][textureIndex] then
					runningTimers[poi] = nil
					runningTimersMax[poi] = nil
					poi.timer:SetTexture(0, 0, 1, 0.5)
					poi.timer.percent = 1
					poi.timer:Update()
				else
					runningTimers[poi] = nil
					runningTimersMax[poi] = nil
					poi.timer.percent = 0
					poi.timer:Update()
				end
			else
				runningTimers[poi] = nil
				runningTimersMax[poi] = nil
				poi.timer.percent = 0
				poi.timer:Update()
			end
		else
			poi:Hide()
			poi.timer.percent = 0
			poi.timer:Update()
		end
	end
	for i = numPois+1, #pois do
		pois[i]:Hide()
	end
end

Cartographer3.Utils.AddEventListener("WORLD_MAP_UPDATE", function(event)
	Cartographer3.Utils.AddTimer(f)
end)

Cartographer3.Utils.AddTimer(function(elapsed, currentTime)
	for poi, time in pairs(runningTimers) do
		runningTimers[poi] = time + elapsed
		poi.timer.percent = runningTimers[poi] / runningTimersMax[poi]
		poi.timer:Update()
	end
end)
